@@ -55,7 +55,7 @@ class MyObj(base.WatcherPersistentObject, base.WatcherObject,
 
     @object_base.remotable
     def update_test(self, context=None):
-        if context and context.user == 'alternate':
+        if context and context.user_id == 'alternate':
             self.bar = 'alternate-context'
         else:
             self.bar = 'updated'
@@ -209,7 +209,7 @@ class _TestObject(object):
 
     def test_with_alternate_context(self):
         ctxt1 = context.RequestContext('foo', 'foo')
-        ctxt2 = context.RequestContext(user='alternate')
+        ctxt2 = context.RequestContext(user_id='alternate')
         obj = MyObj.query(ctxt1)
         obj.update_test(ctxt2)
         self.assertEqual('alternate-context', obj.bar)
@@ -410,9 +410,9 @@ expected_object_fingerprints = {
     'Goal': '1.0-93881622db05e7b67a65ca885b4a022e',
     'Strategy': '1.1-73f164491bdd4c034f48083a51bdeb7b',
     'AuditTemplate': '1.1-b291973ffc5efa2c61b24fe34fdccc0b',
-    'Audit': '1.7-19bc991c0b048263df021a36c8624f4d',
-    'ActionPlan': '2.2-3331270cb3666c93408934826d03c08d',
-    'Action': '2.0-1dd4959a7e7ac30c62ef170fe08dd935',
+    'Audit': '1.8-9cadb11a1fbe7200fb32891b37672a62',
+    'ActionPlan': '2.3-dad26bdb60307f461572739463108a59',
+    'Action': '2.1-71073c74c2ecc3a3d7c7103d0072d366',
     'EfficacyIndicator': '1.0-655b71234a82bc7478aff964639c4bb0',
     'ScoringEngine': '1.0-4abbe833544000728e17bd9e83f97576',
     'Service': '1.0-4b35b99ada9677a882c9de2b30212f35',
